part of katana_module;

@immutable
abstract class RerouteConfig {
  const RerouteConfig();
  Map<String, bool Function(BuildContext context)> get value;
}

class LoginRequiredRerouteConfig extends RerouteConfig {
  const LoginRequiredRerouteConfig({
    this.routePath = "/landing",
  });

  final String routePath;

  @override
  Map<String, bool Function(BuildContext context)> get value => {
        routePath: (context) {
          return context.model?.isSignedIn ?? true;
        },
      };
}

class UserDocumentQueryRerouteConfig extends RerouteConfig {
  const UserDocumentQueryRerouteConfig({
    this.routePath = "/landing",
    required this.query,
  });

  final String routePath;
  final ModelQuery query;

  @override
  Map<String, bool Function(BuildContext context)> get value => {
        routePath: (context) {
          final provider = context.model?.documentProvider(query.path);
          if (provider == null) {
            return false;
          }
          final doc = context.read(provider);
          return query.check(doc);
        },
      };
}
